"""
j2y - a super simple Jinja2 templating command line interface
"""

import io
import os
import sys
import hcl
import json
import yaml
import argparse
import platform

from datetime import datetime

from pathlib import Path
from typing import Any, Callable, Dict, List
from jinja2 import Template, Environment, FileSystemLoader

from j2y.util import parse_extra, tty_size, print_stderr
from j2y.filters import registry as filter_registry


# fmt: off
def parse_args(arguments: List[str]) -> argparse.Namespace:
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "template",
        type=Path,
        help="Path to the Jinja template file."
    )
    parser.add_argument(
        "-c",
        "--context",
        type=argparse.FileType("r"),
        default=[],
        action="append",
        help="Path to the input file that serves as source for the render context"
        "of the Jinja template. This argument may be specified multiple times.",
    )
    parser.add_argument(
        "-o",
        "--output",
        type=argparse.FileType("w"),
        default=sys.stdout,
        help="Path to the output file to which the rendered Jinja template is written"
        "to. If the argument is omitted, j2y will print stdout.",
    )
    parser.add_argument(
        "-f",
        "--format",
        choices=loaders().keys(),
        default="yaml",
        help="File format of the input file(s) specified with --context."
        " If multiple input files where specified they need to have the same file"
        " format",
    )
    parser.add_argument(
        "-x",
        "--extra",
        action="append",
        default=[],
        help="Provide an additional variable for the Jinja template render context."
        " The value of this argument needs to provided as key=value pair."
        " This argument may be specified multiple times."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Print the Jinja template render context prior to the rendered template."
        " The render context is printed to stdout."
    )
    return parser.parse_args(arguments)
# fmt: on


def get_template(path: Path, env: Environment) -> Template:
    return env.get_template(str(path))


def load_contexts(fps: List[io.TextIOWrapper], loader: Callable) -> Dict[str, Any]:
    context: Dict = {}
    for fp in fps:
        context.update(loader(fp) or {})
    return context


def loaders() -> Dict[str, Callable]:
    return {
        "yaml": lambda fp: yaml.load(fp.read()),
        "json": lambda fp: json.load(fp),
        "hcl": lambda fp: hcl.load(fp),
    }


def get_loader(format: str) -> Callable:
    return loaders()[format]


def create_environment(path: Path) -> Environment:
    env = Environment(loader=FileSystemLoader(str(path.absolute())))
    env.filters.update(filter_registry)
    return env


def render_template(template: Path, env: Environment, context: Dict[str, Any]) -> str:
    tpl = get_template(template, env)
    return tpl.render(**context)


def write(output: str, fd: io.TextIOWrapper) -> None:
    fd.write(output)
    fd.write("\n")
    fd.flush()


def default_context() -> Dict[str, Any]:
    return {
        "meta": {
            "date": datetime.utcnow(),
            "platform": dict(platform.uname()._asdict()),
        },
        "env": dict(os.environ),
    }


def print_context(ctx: Dict[str, Any], width: int = 12) -> None:
    print_stderr(yaml.dump(ctx, default_flow_style=False, indent=2, width=width))
    print_stderr("=" * width)
    print_stderr("")


def entrypoint(args: argparse.Namespace) -> None:
    extra = parse_extra(args.extra)
    env = create_environment(Path(os.path.curdir))
    ctx = default_context()

    contexts = args.context
    if not contexts:
        print_stderr("stdin as default value for --context argument is deprecated.")
        print_stderr("Please specify -c or --context explicitly, e.g. `-c -`")
        contexts = [sys.stdin]

    ctx.update(load_contexts(contexts, get_loader(args.format)))
    ctx.update(extra)
    if args.verbose:
        print_context(ctx, tty_size()[0])
    write(render_template(args.template, env, ctx), args.output)


def main() -> None:
    entrypoint(parse_args(sys.argv[1:]))



import json
import yaml
import hashlib
from base64 import urlsafe_b64decode, urlsafe_b64encode

UTF8 = "utf-8"


registry = {
    "json": json.dumps,
    "yaml": yaml.dump,
    "b64encode": lambda a: urlsafe_b64encode(a.encode(UTF8)).decode(UTF8),
    "b64decode": lambda a: urlsafe_b64decode(a.encode(UTF8)).decode(UTF8),
    "md5sum": lambda a: hashlib.md5(a.encode(UTF8)).hexdigest(),
    "sha1sum": lambda a: hashlib.sha1(a.encode(UTF8)).hexdigest(),
    "sha256sum": lambda a: hashlib.sha256(a.encode(UTF8)).hexdigest(),
    "sha512sum": lambda a: hashlib.sha512(a.encode(UTF8)).hexdigest(),
}



import sys
import shutil
import functools

from typing import Dict, List, Tuple, cast


print_stderr = functools.partial(print, file=sys.stderr)


def parse_extra(extra: List[str]) -> Dict[str, str]:
    return dict(cast(Tuple[str, str], x.split("=", 1)) for x in extra)


def tty_size() -> Tuple[int, int]:
    return shutil.get_terminal_size((20, 1))



import os
import io

from pathlib import Path
from setuptools import setup


def read(path):
    path = Path(os.path.dirname(__file__)) / path
    with io.open(path, "r", encoding="utf-8") as f:
        return f.read()


long_description = read("README.rst")
requirements = read("requirements.txt").split("\n")


setup(
    name="j2y",
    url="https://github.com/chaudum/j2y",
    author="Christian Haudum",
    author_email="christian@christianhaudum.at",
    description="Jinja2 Template CLI",
    long_description=long_description,
    platforms=["any"],
    license="Apache License 2.0",
    packages=["j2y"],
    entry_points={"console_scripts": ["j2y = j2y.cli:main"]},
    python_requires=">=3.6",
    install_requires=requirements,
    setup_requires=["setuptools_scm"],
    extras_require={
        "test": [
            "mypy",
            "black==18.6b2",
            "pytest>=3.5",
            "pytest-flake8",
            "pytest-black==0.1.7",
            "pytest-mypy",
        ],
        "docs": ["Sphinx>=1.8,<1.9"],
    },
    use_scm_version=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
